@@ -1,5 +1,5 @@
 /*
-** Copyright (c) 2022-2023 LunarG, Inc.
+** Copyright (c) 2022-2024 LunarG, Inc.
 **
 ** Permission is hereby granted, free of charge, to any person obtaining a
 ** copy of this software and associated documentation files (the "Software"),
@@ -26,90 +26,17 @@
 #include "generated/generated_vulkan_enum_to_json.h"
 #include "decode/descriptor_update_template_decoder.h"
 #include "decode/custom_vulkan_struct_decoders.h"
+
 #include "util/platform.h"
 #include "util/defines.h"
+
 #include "nlohmann/json.hpp"
 #include "vulkan/vulkan.h"
 
 GFXRECON_BEGIN_NAMESPACE(gfxrecon)
 GFXRECON_BEGIN_NAMESPACE(decode)
 
-void FieldToJson(nlohmann::ordered_json& jdata, float data, const JsonOptions& options)
-{
-    if (std::isnan(data))
-    {
-        GFXRECON_LOG_WARNING_ONCE("Converting a NAN to zero.");
-        data = 0.0f;
-    }
-    else if (std::isinf(data))
-    {
-        GFXRECON_LOG_WARNING_ONCE("Converting an infinity to max or min.");
-        if (data < 0)
-        {
-            data = std::numeric_limits<float>::min();
-        }
-        else
-        {
-            data = std::numeric_limits<float>::max();
-        }
-    }
-    // Normal and denormal/subnormal numbers pass through unchanged and unremarked.
-
-    jdata = data;
-}
-
-void HandleToJson(nlohmann::ordered_json& jdata,
-                  const format::HandleId* data,
-                  size_t                  num_elements,
-                  const JsonOptions&      options)
-{
-    if (data)
-    {
-        for (size_t i = 0; i < num_elements; ++i)
-        {
-            HandleToJson(jdata[i], data[i], options);
-        }
-    }
-    else
-    {
-        jdata = nullptr;
-    }
-}
-
-void FieldToJson(nlohmann::ordered_json& jdata, const StringArrayDecoder* data, const JsonOptions& options)
-{
-    if (data && data->GetPointer())
-    {
-        const auto decoded_data = data->GetPointer();
-        for (size_t i = 0; i < data->GetLength(); ++i)
-        {
-            jdata[i] = std::string(decoded_data[i]);
-        }
-    }
-}
-
-void FieldToJson(nlohmann::ordered_json& jdata, const StringDecoder* data, const JsonOptions& options)
-{
-    if (data && data->GetPointer())
-    {
-        const auto decoded_data = data->GetPointer();
-        jdata                   = std::string(decoded_data);
-    }
-}
-
-void FieldToJson(nlohmann::ordered_json& jdata, const StringDecoder& data, const JsonOptions& options)
-{
-    FieldToJson(jdata, &data, options);
-}
-
-void FieldToJson(nlohmann::ordered_json& jdata, const WStringDecoder* data, const JsonOptions& options)
-{
-    if (data && data->GetPointer())
-    {
-        const auto decoded_data = data->GetPointer();
-        jdata                   = std::wstring(decoded_data);
-    }
-}
+using util::JsonOptions;
 
 void FieldToJson(nlohmann::ordered_json&                               jdata,
                  VkGeometryTypeKHR                                     discriminant,
@@ -273,18 +200,6 @@ void FieldToJson(nlohmann::ordered_json&                         jdata,
     }
 }
 
-void FieldToJson(nlohmann::ordered_json& jdata, const Decoded_SECURITY_ATTRIBUTES* data, const JsonOptions& options)
-{
-    if (data && data->decoded_value)
-    {
-        const auto& decoded_value = *data->decoded_value;
-        const auto& meta_struct   = *data;
-        jdata["bInheritHandle"]   = static_cast<bool>(decoded_value.bInheritHandle);
-        FieldToJson(jdata["nLength"], decoded_value.nLength, options);
-        FieldToJson(jdata["lpSecurityDescriptor"], meta_struct.lpSecurityDescriptor->GetAddress(), options);
-    }
-}
-
 void FieldToJson(nlohmann::ordered_json& jdata, const Decoded_VkDescriptorImageInfo* data, const JsonOptions& options)
 {
     if (data && data->decoded_value)
@@ -330,8 +245,11 @@ void FieldToJson(nlohmann::ordered_json& jdata, const Decoded_VkWriteDescriptorS
             case VK_DESCRIPTOR_TYPE_STORAGE_TEXEL_BUFFER:
                 HandleToJson(jdata["pTexelBufferView"], &meta_struct.pTexelBufferView, options);
                 break;
-            case VK_DESCRIPTOR_TYPE_INLINE_UNIFORM_BLOCK:
             case VK_DESCRIPTOR_TYPE_ACCELERATION_STRUCTURE_KHR:
+                // Nothing to do here for acceleration structures as the rest of the data is stored
+                // in the pNext chain
+                break;
+            case VK_DESCRIPTOR_TYPE_INLINE_UNIFORM_BLOCK:
             case VK_DESCRIPTOR_TYPE_ACCELERATION_STRUCTURE_NV:
             case VK_DESCRIPTOR_TYPE_MUTABLE_EXT:
                 GFXRECON_LOG_WARNING("Descriptor type not supported at " __FILE__ ", line: %d.", __LINE__);
@@ -404,6 +322,18 @@ void FieldToJson(nlohmann::ordered_json&                 jdata,
     }
 }
 
+void FieldToJson(nlohmann::ordered_json& jdata, const Decoded_SECURITY_ATTRIBUTES* data, const JsonOptions& options)
+{
+    if (data && data->decoded_value)
+    {
+        const auto& decoded_value = *data->decoded_value;
+        const auto& meta_struct   = *data;
+        jdata["bInheritHandle"]   = static_cast<bool>(decoded_value.bInheritHandle);
+        FieldToJson(jdata["nLength"], decoded_value.nLength, options);
+        FieldToJson(jdata["lpSecurityDescriptor"], meta_struct.lpSecurityDescriptor->GetAddress(), options);
+    }
+}
+
 void FieldToJson(nlohmann::ordered_json&                  jdata,
                  const Decoded_VkPipelineCacheCreateInfo* data,
                  const JsonOptions&                       options)
@@ -422,50 +352,40 @@ void FieldToJson(nlohmann::ordered_json&                  jdata,
     }
 }
 
-template <>
-void FieldToJson(nlohmann::ordered_json&                   jdata,
-                 const PointerDecoder<uint64_t, uint64_t>& data,
-                 const JsonOptions&                        options)
-{
-    if (data.GetPointer())
-    {
-        const auto decoded_value = data.GetPointer();
-        const auto length        = data.GetLength();
-        if (length > 1)
-        {
-            for (size_t i = 0; i < length; ++i)
-            {
-                jdata[i] = decoded_value[i];
-            }
-        }
-        else
-        {
-            jdata = *decoded_value;
-        }
-    }
-}
-
 void FieldToJson(nlohmann::ordered_json&                      jdata,
                  const DescriptorUpdateTemplateDecoder* const pData,
                  const JsonOptions&                           options)
 {
     if (pData)
     {
-        if (pData->GetImageInfoCount())
+        const size_t                        image_info_count = pData->GetImageInfoCount();
+        std::vector<nlohmann::ordered_json> image_infos(image_info_count);
+
+        for (size_t image_info_index = 0; image_info_index < image_info_count; ++image_info_index)
         {
-            FieldToJson(jdata["imageInfos"], pData->GetImageInfoMetaStructPointer(), options);
+            FieldToJson(
+                image_infos[image_info_index], pData->GetImageInfoMetaStructPointer() + image_info_index, options);
         }
-        if (pData->GetBufferInfoCount())
+        jdata["imageInfos"] = image_infos;
+
+        const size_t                        buffer_info_count = pData->GetBufferInfoCount();
+        std::vector<nlohmann::ordered_json> buffer_infos(buffer_info_count);
+
+        for (size_t buffer_info_index = 0; buffer_info_index < buffer_info_count; ++buffer_info_index)
         {
-            FieldToJson(jdata["bufferInfos"], pData->GetBufferInfoMetaStructPointer(), options);
+            FieldToJson(
+                buffer_infos[buffer_info_index], pData->GetBufferInfoMetaStructPointer() + buffer_info_index, options);
         }
-        const auto texel_buffer_view_count = pData->GetTexelBufferViewCount();
+        jdata["bufferInfos"] = buffer_infos;
+
+        const size_t texel_buffer_view_count = pData->GetTexelBufferViewCount();
         if (texel_buffer_view_count > 0)
         {
             HandleToJson(
                 jdata["bufferViews"], pData->GetTexelBufferViewHandleIdsPointer(), texel_buffer_view_count, options);
         }
-        const auto acceleration_structure_count = pData->GetAccelerationStructureKHRCount();
+
+        const size_t acceleration_structure_count = pData->GetAccelerationStructureKHRCount();
         if (acceleration_structure_count > 0)
         {
             HandleToJson(jdata["accelStructViews"],
