@@ -32,6 +32,8 @@ GFXRECON_BEGIN_NAMESPACE(decode)
 const std::vector<std::string> kLoaderLibNames = {
 #if defined(WIN32)
     "vulkan-1.dll"
+#elif defined(__APPLE__)
+    "libvulkan.dylib", "libvulkan.1.dylib", "libMoltenVK.dylib"
 #else
     "libvulkan.so", "libvulkan.so.1"
 #endif
