@@ -393,6 +393,13 @@ VkResult VulkanVirtualSwapchain::CreateSwapchainResourceData(const DeviceInfo*
             VK_IMAGE_LAYOUT_UNDEFINED                                           // initialLayout
         };
 
+        // If compression control structure was present, pass it along to the image
+        // create info.
+        if (swapchain_info->compression_control)
+        {
+            image_create_info.pNext = swapchain_info->compression_control.get();
+        }
+
         if ((swapchain_info->image_flags & VK_SWAPCHAIN_CREATE_MUTABLE_FORMAT_BIT_KHR) ==
             VK_SWAPCHAIN_CREATE_MUTABLE_FORMAT_BIT_KHR)
         {
