@@ -340,6 +340,17 @@ class D3D12CaptureManager : public CaptureManager
                                                            REFIID                          riid,
                                                            void**                          resource);
 
+    void PostProcess_ID3D12Device10_CreateReservedResource2(ID3D12Device10_Wrapper*         wrapper,
+                                                            HRESULT                         result,
+                                                            const D3D12_RESOURCE_DESC*      desc,
+                                                            D3D12_BARRIER_LAYOUT            initial_layout,
+                                                            const D3D12_CLEAR_VALUE*        optimized_clear_value,
+                                                            ID3D12ProtectedResourceSession* protected_session,
+                                                            UINT32                          num_castable_formats,
+                                                            const DXGI_FORMAT*              castable_formats,
+                                                            REFIID                          riid,
+                                                            void**                          resource);
+
     void PreProcess_ID3D12Device3_OpenExistingHeapFromAddress(ID3D12Device3_Wrapper* wrapper,
                                                               const void*            address,
                                                               REFIID                 riid,
@@ -377,6 +388,19 @@ class D3D12CaptureManager : public CaptureManager
                                                             REFIID                          riid,
                                                             void**                          resource);
 
+    void PostProcess_ID3D12Device10_CreateCommittedResource3(ID3D12Device10_Wrapper*         wrapper,
+                                                             HRESULT                         result,
+                                                             const D3D12_HEAP_PROPERTIES*    heap_properties,
+                                                             D3D12_HEAP_FLAGS                heap_flags,
+                                                             const D3D12_RESOURCE_DESC1*     desc,
+                                                             D3D12_BARRIER_LAYOUT            initial_layout,
+                                                             const D3D12_CLEAR_VALUE*        optimized_clear_value,
+                                                             ID3D12ProtectedResourceSession* protected_session,
+                                                             UINT32                          num_castable_formats,
+                                                             const DXGI_FORMAT*              castable_formats,
+                                                             REFIID                          riid,
+                                                             void**                          resource);
+
     void PostProcess_ID3D12Device8_CreatePlacedResource1(ID3D12Device8_Wrapper*      wrapper,
                                                          HRESULT                     result,
                                                          ID3D12Heap*                 heap,
@@ -387,6 +411,18 @@ class D3D12CaptureManager : public CaptureManager
                                                          REFIID                      riid,
                                                          void**                      resource);
 
+    void PostProcess_ID3D12Device10_CreatePlacedResource2(ID3D12Device10_Wrapper*     wrapper,
+                                                          HRESULT                     result,
+                                                          ID3D12Heap*                 heap,
+                                                          UINT64                      heap_offset,
+                                                          const D3D12_RESOURCE_DESC1* desc,
+                                                          D3D12_BARRIER_LAYOUT        initial_layout,
+                                                          const D3D12_CLEAR_VALUE*    optimized_clear_value,
+                                                          UINT32                      num_castable_formats,
+                                                          const DXGI_FORMAT*          castable_formats,
+                                                          REFIID                      riid,
+                                                          void**                      resource);
+
     void PostProcess_ID3D12Resource_Map(
         ID3D12Resource_Wrapper* wrapper, HRESULT result, UINT subresource, const D3D12_RANGE* read_range, void** data);
 
@@ -598,6 +634,18 @@ class D3D12CaptureManager : public CaptureManager
                                                           REFIID                          riid_resource,
                                                           void**                          ppv_resource);
 
+    HRESULT OverrideID3D12Device10_CreateCommittedResource3(ID3D12Device10_Wrapper*         wrapper,
+                                                            const D3D12_HEAP_PROPERTIES*    heap_properties,
+                                                            D3D12_HEAP_FLAGS                heap_flags,
+                                                            const D3D12_RESOURCE_DESC1*     desc,
+                                                            D3D12_BARRIER_LAYOUT            initial_layout,
+                                                            const D3D12_CLEAR_VALUE*        optimized_clear_value,
+                                                            ID3D12ProtectedResourceSession* protected_session,
+                                                            UINT32                          num_castable_formats,
+                                                            const DXGI_FORMAT*              castable_formats,
+                                                            REFIID                          riid_resource,
+                                                            void**                          ppv_resource);
+
     HRESULT OverrideID3D12Device_CreateHeap(ID3D12Device_Wrapper*  wrapper,
                                             const D3D12_HEAP_DESC* desc,
                                             REFIID                 riid,
@@ -685,6 +733,10 @@ class D3D12CaptureManager : public CaptureManager
     bool AddFillMemoryResourceValueCommand(
         const std::map<uint64_t, Dx12ResourceValueAnnotator::Dx12FillCommandResourceValue>& resource_values);
 
+    void SetAgsVersion(int ags_version) { ags_version_ = ags_version; }
+
+    int GetAgsVersion() { return ags_version_; }
+
   protected:
     D3D12CaptureManager();
 
@@ -710,7 +762,6 @@ class D3D12CaptureManager : public CaptureManager
                                       D3D12_RESOURCE_DIMENSION dimension,
                                       D3D12_TEXTURE_LAYOUT     layout,
                                       UINT64                   width,
-                                      UINT64                   size,
                                       D3D12_HEAP_TYPE          heap_type,
                                       D3D12_CPU_PAGE_PROPERTY  page_property,
                                       D3D12_MEMORY_POOL        memory_pool,
@@ -744,6 +795,7 @@ class D3D12CaptureManager : public CaptureManager
     DxgiDispatchTable  dxgi_dispatch_table_;  ///< DXGI dispatch table for functions retrieved from the DXGI DLL.
     D3D12DispatchTable d3d12_dispatch_table_; ///< D3D12 dispatch table for functions retrieved from the D3D12 DLL.
     AgsDispatchTable   ags_dispatch_table_;   ///< ags dispatch table for functions retrieved from the AGS DLL.
+    int                ags_version_{};        ///< ags version.
     static thread_local uint32_t call_scope_; ///< Per-thread scope count to determine when an intercepted API call is
                                               ///< being made directly by the application.
     bool debug_layer_enabled_;                ///< Track if debug layer has been enabled.
