@@ -279,6 +279,17 @@ inline void CreateWrappedHandle<PhysicalDeviceWrapper, NoParentWrapper, DeviceWr
     CreateWrappedDispatchHandle<PhysicalDeviceWrapper, DeviceWrapper>(VK_NULL_HANDLE, handle, get_id);
 }
 
+template <>
+inline void CreateWrappedHandle<DeviceWrapper, NoParentWrapper, DeviceMemoryWrapper>(VkDevice device,
+                                                                                     NoParentWrapper::HandleType,
+                                                                                     VkDeviceMemory* handle,
+                                                                                     PFN_GetHandleId get_id)
+{
+    CreateWrappedNonDispatchHandle<DeviceMemoryWrapper>(handle, get_id);
+    auto memory_wrapper           = GetWrapper<DeviceMemoryWrapper>(*handle);
+    memory_wrapper->parent_device = GetWrapper<DeviceWrapper>(device);
+}
+
 template <>
 inline void CreateWrappedHandle<DeviceWrapper, NoParentWrapper, QueueWrapper>(
     VkDevice parent,
@@ -423,6 +434,7 @@ CreateWrappedHandle<DeviceWrapper, SwapchainKHRWrapper, ImageWrapper>(VkDevice,
         CreateWrappedNonDispatchHandle<ImageWrapper>(handle, get_id);
         wrapper                     = GetWrapper<ImageWrapper>(*handle);
         wrapper->is_swapchain_image = true;
+        wrapper->parent_swapchains.insert(co_parent);
         parent_wrapper->child_images.push_back(wrapper);
     }
 }
@@ -613,8 +625,12 @@ inline void DestroyWrappedHandle<SwapchainKHRWrapper>(VkSwapchainKHR handle)
 
         for (auto image_wrapper : wrapper->child_images)
         {
-            RemoveWrapper<ImageWrapper>(image_wrapper);
-            delete image_wrapper;
+            image_wrapper->parent_swapchains.erase(handle);
+            if (image_wrapper->parent_swapchains.empty())
+            {
+                RemoveWrapper<ImageWrapper>(image_wrapper);
+                delete image_wrapper;
+            }
         }
 
         RemoveWrapper<SwapchainKHRWrapper>(wrapper);
