@@ -77,8 +77,10 @@ bool WriteBmpImage(
         row_pitch = pitch;
     }
 
-    uint32_t image_size = height * row_pitch;
-    if (image_size <= data_size)
+    // BMP image data doesn't require row padding, so compute size with height * width * kImageBpp, not row_pitch *
+    // height.
+    uint32_t bmp_image_size = height * width * kImageBpp;
+    if (bmp_image_size <= data_size)
     {
         FILE*   file   = nullptr;
         int32_t result = util::platform::FileOpen(&file, filename.c_str(), "wb");
@@ -92,7 +94,7 @@ bool WriteBmpImage(
             file_header.reserved1 = 0;
             file_header.reserved2 = 0;
             file_header.off_bits  = sizeof(file_header) + sizeof(info_header);
-            file_header.size      = image_size + file_header.off_bits;
+            file_header.size      = bmp_image_size + file_header.off_bits;
 
             info_header.size             = sizeof(info_header);
             info_header.width            = width;
@@ -137,6 +139,7 @@ bool WritePngImage(
 
 #ifdef GFXRECON_ENABLE_PNG_SCREENSHOT
     uint32_t row_pitch = pitch == 0 ? width * kImageBpp : pitch;
+    stbi_write_png_compression_level = 4;
     if (1 == stbi_write_png(filename.c_str(), width, height, kImageBpp, data, row_pitch))
     {
         success = true;
