@@ -223,7 +223,7 @@ inline size_t GetSystemPageSize()
     return sSysInfo.dwPageSize;
 }
 
-inline void* AllocateRawMemory(size_t aligned_size, bool use_write_watch = false)
+inline void* AllocateRawMemory(size_t aligned_size, bool use_write_watch = false, void* address = nullptr)
 {
     assert(aligned_size > 0);
 
@@ -236,7 +236,7 @@ inline void* AllocateRawMemory(size_t aligned_size, bool use_write_watch = false
             flags |= MEM_WRITE_WATCH;
         }
 
-        return VirtualAlloc(nullptr, aligned_size, flags, PAGE_READWRITE);
+        return VirtualAlloc(address, aligned_size, flags, PAGE_READWRITE);
     }
 
     return nullptr;
@@ -269,9 +269,22 @@ inline pid_t GetCurrentProcessId()
 
 inline uint64_t GetCurrentThreadId()
 {
+#ifdef __APPLE__
+    uint64_t thread_id;
+    pthread_threadid_np(pthread_self(), &thread_id);
+    return thread_id;
+#else
     return static_cast<uint64_t>(syscall(SYS_gettid));
+#endif
 }
 
+#if !defined(__APPLE__)
+inline int SendSignalToThread(uint64_t tid, int signal)
+{
+    return syscall(SYS_tgkill, getpid(), tid, signal);
+}
+#endif
+
 inline void TriggerDebugBreak()
 {
     raise(SIGTRAP);
@@ -419,7 +432,7 @@ inline bool FileSeek(FILE* stream, int64_t offset, FileSeekOrigin origin)
 
 inline size_t FileWriteNoLock(const void* buffer, size_t element_size, size_t element_count, FILE* stream)
 {
-#if defined(__ANDROID__) && (__ANDROID_API__ < 28)
+#if defined(__APPLE__) || (defined(__ANDROID__) && (__ANDROID_API__ < 28))
     return fwrite(buffer, element_size, element_count, stream);
 #else
     return fwrite_unlocked(buffer, element_size, element_count, stream);
@@ -428,7 +441,7 @@ inline size_t FileWriteNoLock(const void* buffer, size_t element_size, size_t el
 
 inline size_t FileReadNoLock(void* buffer, size_t element_size, size_t element_count, FILE* stream)
 {
-#if defined(__ANDROID__) && (__ANDROID_API__ < 28)
+#if defined(__APPLE__) || (defined(__ANDROID__) && (__ANDROID_API__ < 28))
     return fread(buffer, element_size, element_count, stream);
 #else
     return fread_unlocked(buffer, element_size, element_count, stream);
@@ -499,13 +512,14 @@ inline size_t GetSystemPageSize()
 }
 
 // Memory allocation without extra space for private info like with new or malloc
-inline void* AllocateRawMemory(size_t aligned_size, bool use_write_watch = false)
+inline void* AllocateRawMemory(size_t aligned_size, bool use_write_watch = false, void* address = nullptr)
 {
     assert(aligned_size > 0);
 
     if (aligned_size > 0)
     {
-        void* memory = mmap(nullptr, aligned_size, PROT_READ | PROT_WRITE, MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
+        const int flags  = address ? (MAP_PRIVATE | MAP_ANONYMOUS | MAP_FIXED) : (MAP_PRIVATE | MAP_ANONYMOUS);
+        void*     memory = mmap(address, aligned_size, PROT_READ | PROT_WRITE, flags, -1, 0);
 
         if (memory == MAP_FAILED)
         {
@@ -635,6 +649,15 @@ inline T AlignValue(T original)
     return (original + (alignment_t - 1)) & ~(alignment_t - 1);
 }
 
+inline uint64_t SizeTtoUint64(size_t value)
+{
+#if defined(_WIN64)
+    return value;
+#else
+    return static_cast<uint64_t>(value);
+#endif
+}
+
 GFXRECON_END_NAMESPACE(platform)
 GFXRECON_END_NAMESPACE(util)
 GFXRECON_END_NAMESPACE(gfxrecon)
