@@ -103,13 +103,16 @@ const char kMeasurementRangeArgument[]           = "--measurement-frame-range";
 const char kMeasurementFileArgument[]            = "--measurement-file";
 const char kQuitAfterMeasurementRangeOption[]    = "--quit-after-measurement-range";
 const char kFlushMeasurementRangeOption[]        = "--flush-measurement-range";
+const char kFlushInsideMeasurementRangeOption[]  = "--flush-inside-measurement-range";
 const char kSwapchainOption[]                    = "--swapchain";
 const char kEnableUseCapturedSwapchainIndices[] =
     "--use-captured-swapchain-indices"; // The same: util::SwapchainOption::kCaptured
-const char kFormatArgument[]        = "--format";
-const char kIncludeBinariesOption[] = "--include-binaries";
-const char kExpandFlagsOption[]     = "--expand-flags";
-const char kFilePerFrameOption[]    = "--file-per-frame";
+const char kColorspaceFallback[]              = "--use-colorspace-fallback";
+const char kOffscreenSwapchainFrameBoundary[] = "--offscreen-swapchain-frame-boundary";
+const char kFormatArgument[]                  = "--format";
+const char kIncludeBinariesOption[]           = "--include-binaries";
+const char kExpandFlagsOption[]               = "--expand-flags";
+const char kFilePerFrameOption[]              = "--file-per-frame";
 #if defined(WIN32)
 const char kApiFamilyOption[]             = "--api";
 const char kDxTwoPassReplay[]             = "--dx12-two-pass-replay";
@@ -124,6 +127,7 @@ enum class WsiPlatform
     kXlib,
     kXcb,
     kWayland,
+    kMetal,
     kDisplay,
     kHeadless
 };
@@ -133,6 +137,7 @@ const char kWsiPlatformWin32[]    = "win32";
 const char kWsiPlatformXlib[]     = "xlib";
 const char kWsiPlatformXcb[]      = "xcb";
 const char kWsiPlatformWayland[]  = "wayland";
+const char kWsiPlatformMetal[]    = "metal";
 const char kWsiPlatformDisplay[]  = "display";
 const char kWsiPlatformHeadless[] = "headless";
 
@@ -318,6 +323,14 @@ static WsiPlatform GetWsiPlatform(const gfxrecon::util::ArgumentParser& arg_pars
             wsi_platform = WsiPlatform::kWayland;
 #else
             GFXRECON_LOG_WARNING("Ignoring wsi option \"%s\", which is not enabled on this system", value.c_str());
+#endif
+        }
+        else if (gfxrecon::util::platform::StringCompareNoCase(kWsiPlatformMetal, value.c_str()) == 0)
+        {
+#if defined(VK_USE_PLATFORM_METAL_EXT)
+            wsi_platform = WsiPlatform::kMetal;
+#else
+            GFXRECON_LOG_WARNING("Ignoring wsi option \"%s\", which is not enabled on this system", value.c_str());
 #endif
         }
         else if (gfxrecon::util::platform::StringCompareNoCase(kWsiPlatformDisplay, value.c_str()) == 0)
@@ -373,6 +386,12 @@ static std::string GetWsiExtensionName(WsiPlatform wsi_platform)
             return VK_KHR_WAYLAND_SURFACE_EXTENSION_NAME;
         }
 #endif
+#if defined(VK_USE_PLATFORM_METAL_EXT)
+        case WsiPlatform::kMetal:
+        {
+            return VK_EXT_METAL_SURFACE_EXTENSION_NAME;
+        }
+#endif
 #if defined(VK_USE_PLATFORM_HEADLESS)
         case WsiPlatform::kHeadless:
         {
@@ -405,6 +424,10 @@ static std::string GetWsiArgString()
     wsi_args += ',';
     wsi_args += kWsiPlatformWayland;
 #endif
+#if defined(VK_USE_PLATFORM_METAL_EXT)
+    wsi_args += ',';
+    wsi_args += kWsiPlatformMetal;
+#endif
 #if defined(VK_USE_PLATFORM_DISPLAY_KHR)
     wsi_args += ',';
     wsi_args += kWsiPlatformDisplay;
@@ -789,6 +812,11 @@ static void GetReplayOptions(gfxrecon::decode::ReplayOptions& options, const gfx
         options.flush_measurement_frame_range = true;
     }
 
+    if (arg_parser.IsOptionSet(kFlushInsideMeasurementRangeOption))
+    {
+        options.flush_inside_measurement_range = true;
+    }
+
     const auto& override_gpu = arg_parser.GetArgumentValue(kOverrideGpuArgument);
     if (!override_gpu.empty())
     {
@@ -867,6 +895,16 @@ GetVulkanReplayOptions(const gfxrecon::util::ArgumentParser&           arg_parse
         }
     }
 
+    if (arg_parser.IsOptionSet(kColorspaceFallback))
+    {
+        replay_options.use_colorspace_fallback = true;
+    }
+
+    if (arg_parser.IsOptionSet(kOffscreenSwapchainFrameBoundary))
+    {
+        replay_options.offscreen_swapchain_frame_boundary = true;
+    }
+
     replay_options.replace_dir = arg_parser.GetArgumentValue(kShaderReplaceArgument);
     replay_options.create_resource_allocator =
         GetCreateResourceAllocatorFunc(arg_parser, filename, replay_options, tracked_object_info_table);
